package C4::Koha;

use strict;
require Exporter;
use C4::Database;

use vars qw($VERSION @ISA @EXPORT);
  
$VERSION = 0.01;
    
@ISA = qw(Exporter);
@EXPORT = qw(&slashifyDate
	     &fixEthnicity
	     &borrowercategories
	     &ethnicitycategories
	     &automembernum
	     $DEBUG); 

use vars qw();
	
my $DEBUG = 0;

sub slashifyDate {
    # accepts a date of the form xx-xx-xx[xx] and returns it in the 
    # form xx/xx/xx[xx]
    my @dateOut = split('-', shift);
    return("$dateOut[2]/$dateOut[1]/$dateOut[0]")
}

sub fixEthnicity($) { 

    my $ethnicity = shift;
    my $dbh=C4Connect;
    my $sth=$dbh->prepare("Select name from ethnicity where code = ?");
    $sth->execute($ethnicity);
    my $data=$sth->fetchrow_hashref;
    $sth->finish;
    $dbh->disconnect;
    return $data->{'name'};
}

sub borrowercategories {
    my $dbh=C4Connect;
    my $sth=$dbh->prepare("Select categorycode,description from categories order by description");
    $sth->execute;
    my %labels;
    my @codes;
    while (my $data=$sth->fetchrow_hashref){
      push @codes,$data->{'categorycode'};
      $labels{$data->{'categorycode'}}=$data->{'description'};
    }
    $sth->finish;
    $dbh->disconnect;
    return(\@codes,\%labels);
}

sub ethnicitycategories {
    my $dbh=C4Connect;
    my $sth=$dbh->prepare("Select code,name from ethnicity order by name");
    $sth->execute;
    my %labels;
    my @codes;
    while (my $data=$sth->fetchrow_hashref){
      push @codes,$data->{'code'};
      $labels{$data->{'code'}}=$data->{'name'};
    }
    $sth->finish;
    $dbh->disconnect;
    return(\@codes,\%labels);
}

sub automembernum {
  my $dbh=C4Connect;
  my $query="select max(substring(borrowers.cardnumber,2,7)) from borrowers";
  my $sth=$dbh->prepare($query);
  $sth->execute; 
  my $data=$sth->fetchrow_hashref;
  my $cardnumber=$data->{'max(substring(borrowers.cardnumber,2,7))'};
  $sth->finish; 
  $dbh->disconnect;                   
  # purpose: generate checksum'd member numbers.                        
  # We'll assume we just got the max value of digits 2-8 of member 
  #'s from the database and our job is to 
  # increment that by one, determine the 1st and 9th digits and return the full string.
  my @weightings = (8,4,6,3,5,2,1);
  my $sum;
  my $i = 0;  
  if (! $cardnumber) {
    # If DB has no values, start at 1000000
    $cardnumber = 1000000;
  } 
  else { 
    $cardnumber = $cardnumber + 1; 
  } 
  while ($i <8) {
    # step from char 1 to 7.                  
    my $temp1 = $weightings[$i];
    # read weightings, left to right, 1 char at a time
    my $temp2 = substr($cardnumber,$i,1);
    # sequence left to right, 1 char at a time
    #print "$temp2<br>";
    $sum = $sum + ($temp1*$temp2);
    # mult each char 1-7 by its corresponding weighting
    $i++;
    # increment counter
  }
  my $rem = ($sum%11);
  # remainder of sum/11 (eg. 9999999/11, remainder=2)
  if ($rem == 10) {                     
    # if remainder is 10, use X instead
    $rem = "X";
  }
  $cardnumber="V$cardnumber$rem"; 
  return($cardnumber);
}
1;
__END__

=head1 NAME

Koha - Perl Module containing convenience functions for Koha scripts

=head1 SYNOPSIS

  use Koha;


  $date = slashifyDate("01-01-2002")
  $ethnicity=fixEthnicity('asian');
  ($categories,$labels)=borrowercategories();

=head1 DESCRIPTION

Koha.pm provides many functions for Koha scripts.

slashifyDate() takes a dash separated date string and returns a slash 
separated date string

=head1 AUTHOR

Pat Eyler, pate@gnu.org

=head1 SEE ALSO

perl(1).

=cut





